#include "crefl.1.7.1_common.h"

#define	NUM1KMCOLPERSCAN 1354
#define	NUM1KMROWPERSCAN 10
#define DEMFILENAME	"tbase.hdf"
#define DEMSDSNAME	"Elevation"
#define ANCPATH		"."

void init_crefl_in_options_st (crefl_in_options_st *options_val_st)
{
    int ib; 
    
    options_val_st->reflmin = REFLMIN;
    options_val_st->reflmax = REFLMAX;
    options_val_st->maxsolz = MAXSOLZ;
    
    options_val_st->output1km = 0;
    options_val_st->output500m = 0;
    options_val_st->append = 0;
    options_val_st->bands = 0;
    options_val_st->gzip = 0;
    options_val_st->nearest = 0;
    options_val_st->overwrite = 0;
    options_val_st->sealevel = 0;
    options_val_st->TOA = 0;
    options_val_st->verbose = 0;
    
    for (ib = 0; ib < NR_FILES; ib++) {
        options_val_st->input_files [ib] = (char *) NULL;
        options_val_st->input_file_id_arr [ib] = -1;
    }
        
    for (ib = 0; ib < Nbands; ib++) {
		options_val_st->process[ib] = FALSE;
    }
}

int modis_parse_bands(char *bandstr, unsigned char process[Nbands])
{
	int band, nb;
	char *ptr, *s;

	nb = 0;
	ptr = bandstr;
	while ((s = strtok(ptr, ","))) {
		band = atoi(s);

		if ((band > 0) && (band < Nbands))
			process[band - 1] = TRUE;
		else
			return -1;

		if (++nb)
			ptr = (char *) NULL;
	}

	return 0;
}

/*
Returns 0 if xmin <= x <= xmax, non-zero otherwise.
*/
int
range_check(float x, float xmin, float xmax)
{
	return (x < xmin) || (x > xmax);
}

void
usage(void)
{
	fputs("Usage:\n", stderr);
	fputs
	    ("crefl <modis | viirs> [--verbose] [--1km|--500m] [--nearest] [--toa|--sealevel]\n"
	    "      [--gzip] [--maxsolz=angle] [--range=min,max] [--overwrite|--append]\n"
	    "      [--bands=<band1,band2,band3,...>] --of=<output file>\n"
	    "      <MOD021KM|MOD02CRS|MOD09CRS file> [<MOD02HKM file>] [<MOD02QKM file>]\n",
	    stderr);

	fprintf(stderr, "Version %s, compiled %s %s.\n",
	    PROCESS_VERSION_NUMBER, __DATE__, __TIME__);
}

void dem_init_data (crefl_in_options_st *options_val_st, SDS *dem_sds_st)
{
    char *ancpath;
    char dummy[H4_MAX_NC_NAME];
    static char dem_filename_buff [MAXNAMELENGTH];
    
    if (!options_val_st->sealevel && !options_val_st->TOA) {
		dem_sds_st->filename = dem_filename_buff;

		if ((ancpath = getenv("ANCPATH")) == NULL)
			sprintf(dem_sds_st->filename, "%s/%s", ANCPATH, DEMFILENAME);
		else
			sprintf(dem_sds_st->filename, "%s/%s", ancpath, DEMFILENAME);

		if ((dem_sds_st->file_id = SDstart(dem_sds_st->filename, DFACC_READ)) == -1) {
			fprintf(stderr, "Cannot open file %s.\n",
			    dem_sds_st->filename);
			exit(1);
		}
	}

    if (options_val_st->sealevel || options_val_st->TOA) {
		dem_sds_st->id = -1;
		dem_sds_st->Nl = dem_sds_st->Np = 0;
	} else {
		/* dem_sds_st->name = strdup(DEMSDSNAME); */
		dem_sds_st->name = DEMSDSNAME;

		if ((dem_sds_st->index = SDnametoindex(dem_sds_st->file_id, dem_sds_st->name)) == -1) {
			fprintf(stderr, "Cannot find SDS %s in file %s.\n",
			    dem_sds_st->name, dem_sds_st->filename);
			exit(1);
		}

		if ((dem_sds_st->id = SDselect(dem_sds_st->file_id, dem_sds_st->index)) == -1) {
			fprintf(stderr, "Cannot select SDS no. %d\n",
			    dem_sds_st->index);
			exit(1);
		}
		if (SDgetinfo
		    (dem_sds_st->id, dummy, &dem_sds_st->rank, dem_sds_st->dim_sizes, &dem_sds_st->num_type,
			&dem_sds_st->n_attr) == -1) {
			fprintf(stderr,
			    "Can't get info from SDS \"%s\" in file %s.\n",
			    dem_sds_st->name, dem_sds_st->filename);
			SDendaccess(dem_sds_st->id);
			exit(1);
		}

		dem_sds_st->Nl = dem_sds_st->dim_sizes[0];
		dem_sds_st->Np = dem_sds_st->dim_sizes[1];
		dem_sds_st->rowsperscan =
		    (int) (NUM1KMROWPERSCAN * dem_sds_st->Np /
		    (float) NUM1KMCOLPERSCAN + 0.5);
	}

}

void dem_finalize_data (crefl_in_options_st *options_val_st, SDS *dem_sds_st)
{
    /* not allocated if --sealevel specified */
	if (dem_sds_st->data) {
        free (dem_sds_st->data);
    }
}

